//package urdar;
import java.io.*;

/** This class contains the <code>main</code>-method. All parameters of the simulation
 * are controlled via the file <code>settings.txt</code>.
 * @author Philip Gerlee
 * @author Torbjorn Lundh
 * @version 0.1 1st Jan 2011
 */
public class Simulation {
	
	public static void main (String arg []) throws IOException, FileNotFoundException {
		int divisions = 0,tmax = 0,popSize = 0,poolSize = 0,stringLength = 0,entLevel = 0,smode = 0;
		boolean rmode = false, logSpecies = false, logInts = false, logEff = false;
		double flow = 0,flipProb = 0,mutRate = 0;
		boolean[] bitString = null;
		String row = "empty";
		String[] contents;
		World W;
		//Reading settings from the file specified in arg[0]
		try {
			BufferedReader settings = new BufferedReader(new FileReader(arg[0]));		
			while (row != null) {
				row = settings.readLine();
				if (row != null) {
					if (row.length() > 0) {
						if (!row.substring(0,1).equals("#")) {
							contents = row.split(" ");
							if (contents[0].equals("NO_UPDATES")){
								tmax = Integer.parseInt(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("POP_SIZE")){
								popSize = Integer.parseInt(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("POOL_SIZE")){
								poolSize = Integer.parseInt(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("STRING_SIZE")){
								stringLength = Integer.parseInt(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("FLOW")){
								flow = Double.parseDouble(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("ENT_LEVEL")){
								entLevel = Integer.parseInt(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("RAND_MODE")){
								rmode = contents[1].equals("1");
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("FLIP_PROB")){
								flipProb = Double.parseDouble(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("STRING_MODE")&&rmode){
								smode = Integer.parseInt(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("STRING")&&!rmode){
								System.out.print(contents[0] + "  ");
								bitString = new boolean[contents[1].length()];
								for (int i = 0; i < contents[1].length()-1; i++) {
									bitString[i] = contents[1].substring(i,i+1).equals("1") ? true : false;
									System.out.print(contents[1].substring(i,i+1));
								}
								System.out.println();

							}
							if (contents[0].equals("MUT_RATE")){
								mutRate = Double.parseDouble(contents[1]);
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("LOG_SPECIES")){
								logSpecies = contents[1].equals("1");
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("LOG_INTERACTIONS")){
								logInts = contents[1].equals("1");
								System.out.println(contents[0] + "  " + contents[1]);
							}
							if (contents[0].equals("LOG_EFFICIENCY")){
								logEff = contents[1].equals("1");
								System.out.println(contents[0] + "  " + contents[1]);
							}
						}
					}
				}
			}
		} catch(Exception e) {
			System.err.println("Error: " + e.getMessage());
		}
		if (rmode) {
			//Creates world with random food
			W = new World(popSize,poolSize,stringLength,flow,flipProb,smode,entLevel); 
		}
		else {
			//Creates world with specified inflowing strings
			W = new World(popSize,poolSize,flow,bitString,entLevel);
		}
		W.setMutRate(mutRate);
		//W.Inoculate(new int[] {109,90},new double[] {0.5,0.5});
		
		//Updating the system for tmax time steps
		for (int i = 0; i < tmax; i++) {
			W.update();
			if (i%100==0) {
				if (logSpecies) {
					W.saveState("rules.txt");
				}
				if (logEff) {
					W.saveEff("eta.txt");
				}
				System.out.println("Time:   "  + i + "\t" + "Generation:   " + (double) divisions/popSize);
				divisions = W.getDivides();
			}			      
		}
		if (logInts) {
			W.saveInts("ints.txt");
		}

	}
}


